<?php

/**
 * content actions.
 *
 * @package    customerWeb
 * @subpackage content
 * @author     Per-Kristian Helland, Åshild K. Thorrud
 * @version    SVN: $Id: actions.class.php 23810 2009-11-12 11:07:44Z Kris.Wallsmith $
 */
class contentActions extends sfActions
{
 /**
  * Executes index action
  *
  * @param sfRequest $request A request object
  */
  public function executeIndex(sfWebRequest $request)
  {
    
  }

  /**
     * Must be here in order to show the page /comparePeriods, i.e.
     * "Mitt forbruk"
     */
  public function executeSpecificDetails()
  {
      
  }

  public function executeTest()
  {
      
  }

  /**
     * This method handles the AJAX-request when opening the "Historikk" page.
     * If you want to compare by month, it fetches data with the method
     * getMeterDataInXMonthsIntervals, and if you want to compare by weeks, 
     * getMeterDataWeeks is used. Both of them can be found in MeterDataTable.class.php
     *
     * @return json data
     */
  public function executeFetchDataComparePeriods()
  {
      $type = isset ($_POST["type"]) && is_numeric($_POST["type"])? $_POST["type"] : 0;
      if($type == 1)
      {
          $dataenMin = MeterDataTable::getMeterDataWeeks(1);
      }
      else if($type == 2)
      {
          $dataenMin = MeterDataTable::getMeterDataInXMonthsIntervals(1, $type);
      }
      $this->getResponse()->setContent(json_encode($dataenMin));
      return sfView::NONE;
  }
  
  /**
     * This method lets us poll the meter for active power in real time from the web
     * page. This is done by creating a socket and connecting to the Java ServerSocket
     * which listens for connections. It sends the string "FetchData" and gets back 
     * either the active power or the time left before it is OK to poll the meter again.
     *
     * @return json data
     */
  public function executePollMeter()
  {
        $service_port = "1337"; //getservbyname(‘www’, ‘tcp’);
        $address = "127.0.0.1"; //gethostbyname(‘www.example.com’);
        $socket = socket_create(AF_INET,SOCK_STREAM, SOL_TCP);
        if ($socket === false) {
                echo "socket_create() failed: reason: " . socket_strerror(socket_last_error()) . "<br />";
        }
        $result = socket_connect($socket, $address, $service_port);
        if ($result === false) {
                echo "socket_connect() failed.\nReason: ($result) " . socket_strerror(socket_last_error($socket)) . "<BR>";
        }
        $in = "FetchData";
        $in .= chr(13) . chr(10); //End Of Java Transmission 
        $out = '';
        $response ='';
        //echo “Sending Data Transfer …”;
        socket_write($socket, $in, strlen($in));
        //echo “Reading response:\n\n<BR>”;
        try {
            while ($out = @socket_read($socket,1337, PHP_NORMAL_READ)) {
                    $response .= $out;
            }
        } catch (Exception $e) {
            echo "Caught exception: ".  $e->getMessage(). "\n";
        }
        //echo “<BR><BR>Closing socket…”;
        socket_close($socket);
        $this->getResponse()->setContent(json_encode($response));
        return sfView::NONE;
  }
  
  /**
     * Must be here in order to show the page /comparePeriods, i.e.
     * "Historikk"
     */
  public function executeComparePeriods()
  {
      
  }

  /**
     * This method handles the "Mitt forbruk" section of the page.
     * Depending on how long the time period specified is, it fetches
     * an "appropriate" amount of data and returns it as json data.
     *
     * @return json data
     */
  public function executeConsumptionIntervalSpecified()
  {
      $interval = is_numeric($_POST['interval']) ? $_POST['interval'] : 15;
      $start = $_POST["start"];
      $end = $_POST["end"];
      $timeDiff = MeterDataTable::findTimeDifference($start, $end);
//      echo $timeDiff . "###";
      if($timeDiff == 'quarter') $interval = 0.25;
      else if($timeDiff == 'hour') $interval = 1;
      else if($timeDiff == 'day') $interval = 24;
      else if($timeDiff == 'week') $interval = 24*30;
      else $interval = 15;
      
      $start = date($start).":00";
      $end = date($end).":00";
      $dataenMin = MeterDataTable::getMeterDataInXMinutesIntervals(1, $interval, $start, $end);
      $meters = MeterDataTable::getMeterDataForTimePeriod(1, $start, $end, "quarter");
      $result = array();
        foreach ($meters as $meter) {
            $time = (strtotime($meter["time"]) + 3600) * 1000;
            $result[] = array($time, $meter["record"]);
        }
      $this->getResponse()->setContent(json_encode($result));
      return sfView::NONE;
  }

    /**
     * This method fetches all the information that is displayed in
     * the main graph in the "Hjem" page. Everything is stacked up in arrays
     * and returned as json data which is then parsed by the receiving browser.
     * Data for three consumption bar graphs, three cumulative line graphs and
     * two price line graphs are gathered.
     *
     * @return json data
     */
  public function executeFetchAllAvailableInformation()
  {
        $results = array();

        // Fetch consumption bar charts
        $hours = array(24,168, 729);
        $interval = 60;
        $time = new DateTime();
        $now = date('Y-m-d H:i:s');
        foreach ($hours as $h)
        {
            date_sub($time, date_interval_create_from_date_string(floor($h) . " hours"));
            $thePast = $time->format('Y-m-d H:i:s');
            $results[] = MeterDataTable::getMeterDataInXMinutesIntervals(1, $interval, $thePast, $now);
        }

        // Fetch consumption linegraph
        foreach ($hours as $h)
        {
            $results[] = MeterDataTable::getMeterDataForLastHours(1, $h);
        }

        // Fetch price information for the future
        foreach ($hours as $h)
        {
            $results[] = PriceTable::getPriceForLastHours($h, false);
        }

        // Fetch price information for the past
        foreach ($hours as $h)
        {
            $results[] = PriceTable::getPriceForLastHours($h, true);
        }
       
        $this->getResponse()->setContent(json_encode($results));
        return sfView::NONE;

  }

  /**
     * This method fetches weather information from yr.no's weather service.
     * NB: As of now hard coded to get data for Trondheim, Sør-Trøndelag!
     *
     * @return json data
     */
  public function executeTemperatureInformation()
  {
       // Fetch temperature information
      $results = array();
      $hours = array(24,168, 729);
        $file = "http://www.yr.no/sted/Norge/S%F8r-Tr%F8ndelag/Trondheim/Trondheim/varsel.xml";
        $timeSpans = array(
            0 => array(),
            1 => array(),
            2 => array()
        );
        if(file_get_contents( $file )) {
            $content = file_get_contents( $file );
            $xml = simplexml_load_string( $content );
            $i = 0;
            foreach ($xml->forecast->tabular->time as $time)
            {
                for ($j = 0; $j < 3; $j++)
                {
                    if($i == 0){
                        $timeSpans[$j][] = array("time" => ((time() + 3600) * 1000), "temperature" => (int)$time->temperature["value"]);
                        $i++;
                    }
                    if(strtotime($time["from"]) < time() + $hours[$j] * 3600)
                        $timeSpans[$j][] = array("time" => (strtotime($time["from"]) + 3600) * 1000, "temperature" => (int)$time->temperature["value"] );
                }

            }
        }
        foreach ($timeSpans as $tempAtTime)
        {
            $results[] = $tempAtTime;
        }
      $this->getResponse()->setContent(json_encode($results));
      return sfView::NONE;
  }




}
