<?php

/**
 * MeterDataTable
 *
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class MeterDataTable extends Doctrine_Table {

    /**
     * Returns an instance of this class.
     *
     * @return object MeterDataTable
     */
    public static function getInstance() {
        return Doctrine_Core::getTable('MeterData');
    }
    
    /**
     * Returns an array with time and kWh-usage for the gievn interval
     *
     * @return array(unix_time_milli, usage_in_interval)
     */
    public static function getMeterDataInXMinutesIntervals($custId, $interval, $timeStart, $timeEnd)
    {
        if(!is_numeric($interval)) $interval = 15;
        $meters = Doctrine_Core::getTable('MeterData')
                        ->createQuery('m')
                        ->select("COUNT(*) as count, SUM(m.record) as sum, UNIX_TIMESTAMP(m.time) AS time, ROUND( UNIX_TIMESTAMP(m.time) / (".$interval."*60) ) AS timekey")
                        ->where('m.customer_id = ?', $custId)
                        ->andWhere('m.time >= ?', $timeStart)
                        ->andWhere('m.time < ?', $timeEnd)
                        ->groupBy("timekey")
                        ->orderBy('time')
                        ->execute();
        $result = array();
        foreach ($meters as $meter) {
            $time = ($meter->time + 3600) * 1000;
            $result[] = array($time, $meter->sum/$meter->count);
        }
        return $result;
    }
    
    /**
     * Returns the nearest monday which happens before $time
     * represented as datetime object
     *
     * @return datetime $monday
     */
    public static function findMondayBackInTime($time)
    {
        if(date('D', strtotime($time)) == "Mon")
            return $time;
        $monday = "";
        $now = date('Y-m-d h:m:s', strtotime("now")); 
        $some_date = $time;
        $found = 0; 
        $i=0; 
        while(!$found) 
        { 
            $find_monday = date("Y-m-d", strtotime('+'.$i.' monday')); 
            if($some_date > $find_monday ) 
            { 
                $monday = $find_monday; 
                $found = 1; 
                break; 
            } 
            else 
                $i--; 
        }
        return $monday;
    }

    /**
     * This method is used in "Historikk" at the site for the week
     * view. Returns an array of week number adjusted to which year it
     * applies to (-0.3 for two years back, -0.1 for one year and +0.1 weeks
     * for this year.), the usage in that week and the given year.
     *
     * @return array(weekNumber, usage_in_interval, year)
     */
    public static function getMeterDataWeeks($custId)
    {
        $connection = Doctrine_Manager::connection();
        $result = array();
        $dayOnGraph = array(0.1, -0.1, -0.3);
        $year = date('Y');
        $prev = null;
        for($i = 0; $i<3; $i++)
        {
            $relevant_year = ((int)date('Y') + $i - 2);
            $monday_this_week = MeterDataTable::findMondayBackInTime(((int)date('Y') + $i - 2) . date ('-m-d'));
            $conv = "- " . ((2-$i)*52 + 10) . " weeks";
            $start = date("Y-m-d", strtotime($conv));//$relevant_year . date('-m-d 00:00:00', strtotime('-9 weeks Monday'));
            $end = $monday_this_week . " 12:00:00";//$relevant_year . date("-m-d H:i:s");
            $query = "SELECT time, YEAR(time) AS year, MONTH(time) AS month, WEEK(time) AS week, record, COUNT(*)
                        FROM    meter_data
                        WHERE time > '" . $start ."' AND time < '".$end . "' GROUP BY
                        YEAR(time), WEEK(time) WITH ROLLUP";
            $statement = $connection->execute($query);
            $statement->execute();
            $counter = 0;
            $prev = $statement->fetch(PDO::FETCH_OBJ);
            while ($meter = $statement->fetch(PDO::FETCH_OBJ))
            {
                $counter++;
                if($counter > 1)
                {
                    if($meter->week)
                    {
                        $rec = $meter->record - $prev->record;
                        $result[] = array($meter->week + $dayOnGraph[($year - $prev->year)], $rec, $prev->year);
                        $prev = $meter;
                    }
                }
            }
        }
        return $result;
    }

    /**
     * This method is used in "Historikk" at the site for the month
     * view. Returns an array of unix time stamps (milli) adjusted to which year
     * it applies to (day 7 for two years back, day 14 for one year and day 21
     * for this year), the usage in that month and the given year.
     *
     * @return array(month as timestamp, usage_in_month, year)
     */
    public static function getMeterDataInXMonthsIntervals($custId, $type = 0)
    {
        $connection = Doctrine_Manager::connection();
        $result = array();
        $dayOnGraph = array(21,14,7);
        $year = date('Y');
        $prev = null;
        $relevant_year = ((int)date('Y') - 2);
        $start = $relevant_year . date('-1-1 00:00:00');
        $end = date("Y-12-31 23:59:s");
        $query = "SELECT time, YEAR(time) AS year, MONTH(time) AS month, record
                    FROM    meter_data
                    WHERE time > '" . $start ."' AND time < '".$end . "' GROUP BY
                    YEAR(time), MONTH(time) WITH ROLLUP";
        $statement = $connection->execute($query);
        $statement->execute();
        $counter = 0;
        $firstOfMonth = date('Y-m-d H:i:s',strtotime('+0 second',strtotime(date('m').'/01/'.date('Y').' 00:00:00')));
        $now = date("Y-m-d H:i:s");
        $usageThisMonth = MeterDataTable::getConsumptionForTimePeriod($custId, $firstOfMonth, $now);
        $prev = $statement->fetch(PDO::FETCH_OBJ);
        while ($meter = $statement->fetch(PDO::FETCH_OBJ))
        {
            if($meter->year && $meter->month)
            {
                $day = $dayOnGraph[(date('Y')-$prev->year)];
                $datetime = date('Y') . "-" .$prev->month."-". $day." 00:00:00";
                $time = strtotime($datetime)."000";
                $rec = $meter->record - $prev->record; 
                if($meter)
                    $result[] = array($time, $rec, $prev->year);
                $prev = $meter;
            }
            $counter++;
        }
        return $result;
    }

    
    /**
     * This method is used to fetch the meter data for the cumulative view
     * on the front page of the site. It returns an array of all the records in
     * the last hours for a customer with the given timestamp in millisecs attached
     * to it.
     *
     * @return array(unix_milli, record)
     */
    public static function getMeterDataForLastHours($custId, $hours = 0) {
        $time = new DateTime();
        date_sub($time, date_interval_create_from_date_string(floor($hours) . " hours"));
        $thePast = $time->format('Y-m-d H:i:s');
        if($hours > 700) // månedsvisning
        {
            $meters = MeterDataTable::getMeterDataForTimePeriod($custId, $thePast, date('Y-m-d H:i:s'));
            $result = array();
            foreach ($meters as $meter) {
                $rec = array(strtotime($meter["time"]) . "000", $meter["record"]);
                $result[] = $rec;
            }
            return $result;
        }
        else
        {
            $meters = Doctrine_Core::getTable('MeterData')
                        ->createQuery('m')
                        ->select('UNIX_TIMESTAMP(m.time) AS timestamp, m.record')
                        ->where('m.customer_id = ?', $custId)
                        ->andWhere('m.time > ?', $thePast)
                        ->orderBy('m.time')
                        ->execute();
            $result = array();
            foreach ($meters as $meter) {
                $rec = array(($meter->timestamp) . "000", $meter->record);
                $result[] = $rec;
            }
            return $result;
        }
    }
    
    
    /**
     * This method is used fetch the record before a given $time. The first record
     * with timestamp less than $time is returned.
     *
     * @return object record
     */
     public static function getPreviousMeterRecordFromTime($custId, $time) {
        $query = "SELECT * FROM `meter_data` WHERE time < '".$time . "' ORDER BY time DESC LIMIT 1";
        $connection = Doctrine_Manager::connection();    
        $statement = $connection->execute($query);
        $statement->execute();
        return $statement->fetch(PDO::FETCH_OBJ);
    }

    
    /**
     * This method calculates the consumption within the given period by
     * finding the first and last record and returning the difference between
     * them. 
     * 
     * @return double difference
     */
    public static function getConsumptionForTimePeriod($custId, $start, $end)
    {
        $query = "SELECT record FROM `meter_data` WHERE time >= '".$start . "' ORDER BY time LIMIT 1";
        $connection = Doctrine_Manager::connection();    
        $statement = $connection->execute($query);
        $statement->execute();
        $first = $statement->fetch(PDO::FETCH_OBJ);
        if(!$first->record)
        {

        }
        $query = "SELECT * FROM `meter_data` WHERE time < '".$end . "' ORDER BY time DESC LIMIT 1";
        $statement = $connection->execute($query);
        $statement->execute();
        $last = $statement->fetch(PDO::FETCH_OBJ);
        if(!$last->record)
        {
            
        }
        return $last->record - $first->record;
    }
    
    
    /**
     * This method bulks meter data recordings together in groups of months,
     * days, weeks, days, hours or quarters according to what input is given 
     * by $type. Then all values are put into an array containing arrays of
     * time and usage record values for each period. $start and $end gives the time
     * period, while $type defines how small or large the intervals should be
     * 
     * @return array(arrays) timeAndRecord
     */
    public static function getMeterDataForTimePeriod($custId, $start, $end, $type = 'none')
    {
        if($type !== 'none')
            $groupBy = $type;
        else 
            $groupBy = MeterDataTable::getOrderByCoefficient($start, $end);
        
        $meters = array();
        $connection = Doctrine_Manager::connection();
        if($groupBy == 'month')
        {
            $query = "SELECT time, YEAR(time) AS year, MONTH(time) AS month, AVG(record) AS record, COUNT(*)
                        FROM    meter_data
                        WHERE time > '" . $start ."' AND time < '".$end . "' GROUP BY
                        YEAR(time), MONTH(time)";
            $statement = $connection->execute($query);
            $statement->execute();
        }
        else if($groupBy == 'week')
        {
            $query = "SELECT time, YEAR(time) AS year, MONTH(time) AS month, WEEK(time) AS week, AVG(record) AS record, COUNT(*)
                        FROM    meter_data
                        WHERE time > '" . $start ."' AND time < '".$end . "' GROUP BY
                        YEAR(time), MONTH(time), WEEK(time)";
            $statement = $connection->execute($query);
            $statement->execute();
        }
        else if($groupBy == 'day')
        {
            $query = "SELECT time, YEAR(time) AS year, MONTH(time) AS month, DAY(time) AS day, AVG(record) AS record, COUNT(*)
                        FROM    meter_data
                        WHERE time > '" . $start ."' AND time < '".$end . "' GROUP BY
                        YEAR(time), MONTH(time), DAY(time)";
            $statement = $connection->execute($query);
            $statement->execute();
        }
        else if($groupBy === 'hour')
        {
            $query = "SELECT time, YEAR(time) AS year, MONTH(time) AS month, DAY(time) AS day, HOUR(time) AS hour, AVG(record) AS record, COUNT(*)
                        FROM `meter_data`
                        WHERE time > '" . $start ."' AND time < '".$end . "' GROUP BY
                        YEAR(time), MONTH(time), DAY(time), HOUR(time)";
            $statement = $connection->execute($query);
            $statement->execute();
        }
        else if($groupBy == 'quarter')
        {
            $query = "SELECT time, YEAR(time) AS year, MONTH(time) AS month, DAY(time) AS day, HOUR(time) AS hour, MINUTE(time) AS minute, AVG(record) AS record, COUNT(*)
                        FROM    meter_data
                        WHERE time > '" . $start ."' AND time < '".$end . "' GROUP BY
                        YEAR(time), MONTH(time), DAY(time), HOUR(time), MINUTE(time)";
            $statement = $connection->execute($query);
            $statement->execute();
        }
        $startMeter = MeterDataTable::getPreviousMeterRecordFromTime($custId, $start);
        $result = array();
        if(!$startMeter)
           $result[] = array("time" => date("Y-m-d H:i:s",strtotime($start)-1), "record" => $meters[0]->record);
        else
            $result[] = array("time" => $startMeter->time, "record" => $startMeter->record);
        while ($meter = $statement->fetch(PDO::FETCH_OBJ))
        {
            $result[] = array("time" => $meter->time, "record" => $meter->record);
        }
        return $result;
    }

    
    /**
     * Return the last active_power value fetched from the database.
     * 
     * @return double last_active_power
     */
    public static function getCurrentUsage($custId) {
        $now = new DateTime();
        $now = $now->format('Y-m-d H:i:s');
        $usage = Doctrine_Core::getTable('MeterData')
                        ->createQuery('m')
                        ->where('m.customer_id = ?', $custId)
                        ->andWhere('m.time <= ?', $now)
                        ->orderBy('m.time DESC')
                        ->limit(1)
                        ->execute();
        $sum = 0;
        $time0 = strtotime($usage[0]->time);
        return $usage[0]->active_power; //round($diffUsage / $diff, 2);
    }

    public static function compare_eTime($a, $b)
    {
        return strtotime($a[1]) > strtotime($b[1]);
    }
    
    
    /**
     * Helper method that gives the "resolution" to how dense the measures should
     * be. I.e. should records from each quarter be given, or only one for each week?
     * The time difference between $start and $end is the deciding factor as to
     * the result returned. The main reason for this method is performance issues and
     * having the ability to deal with a lot of records. We can't use all meter records
     * when calculating the output in real time. Therefore, large time differences
     * yields more fewer records per time slot.
     * 
     * @return string oneRecordPer____
     */
    public static function getOrderByCoefficient($start, $end)
    {
        $timeDiff = MeterDataTable::findTimeDifference($start, $end);
        if($timeDiff < 86401)
            return 'quarter';
        if($timeDiff < 605000) // week 605000
            return 'hour';
        else if($timeDiff < 18748801) // month 18748801
            return 'day';
        else if($timeDiff < 234985600) // year234985600
            return 'week';
        else
            return 'month';
    }
    
    
    /**
     * This method is used to calculate the consumption in kroner for a customer's
     * usage of electricity in a given period between $start and $end. It gets all
     * the meter data and the prices in the given time range, sorts them according to
     * their timestamp, iterates through each of the events (which is either a 
     * 'meter reading' (AL) or a 'price change' (PE)) and sums up how much is to be
     * paid for the consumption in each little period. This method can be quite time 
     * consuming and resource intensive because of the immense amounts of data it
     * processes and sorts. We have done some effort to optimize it, but more should
     * definitely be done.
     *
     * @return double sum
     */
    public static function getExpenditureInTimePeriod($custId, $start, $end)
    {
        $midnight = $start;
        $now = date('Y-m-d H:i:s');
        if(strtotime($end) < strtotime($now))
            $now = $end;
        $meters = MeterDataTable::getMeterDataForTimePeriod($custId, $midnight, $now);
        $prices = PriceTable::getPriceDataForTimePeriod($midnight, $now);
        $meterCount = count($meters);
        $priceIndex = 0;
        $sum = 0;
        $timeEvents = array();
        foreach ($meters as $m)
        {
            // event / eTime
            $timeEvents[] = array("AL", $m["time"]);
        }
        foreach ($prices as $p)
        {
            $timeEvents[] = array("PE", $p["starttime"]);
        }
        usort($timeEvents, 'MeterDataTable::compare_eTime');
        $i = 0;
        $j = 0;
        $consumptionTimeReached = $meters[0]["time"];
        $lastMeterIndex = 0;
        foreach ($timeEvents as $n => $te)
        {
            if($n > 1)
            {
                if($i == $meterCount)
                {
                    break;
                }
                if($te[0] == "AL")
                {
                    $i++;
                    if($i < $meterCount - 1)
                        goto nextEvent;
                }
                $m = $meters[$i];
                $lastm = $meters[$lastMeterIndex];
                $lastMeterIndex = $i;
                $p = $prices[$j];
                $timeDiff = MeterDataTable::findTimeDifference($consumptionTimeReached, $te[1]);
                $timePeriodLength = MeterDataTable::findTimeDifference($m["time"], $lastm["time"]);
                if($timePeriodLength != 0)
                {
                    $secConsumption = ($m["record"] - $lastm["record"]) / $timePeriodLength;
                }
                else $timeDiff = 0;
                $sum += $timeDiff * $secConsumption * $p["price"];
                $consumptionTimeReached = $te[1];
                if($te[0] == "PE")
                {
                    $j++;
                }
            }
            nextEvent:
        }
        return round($sum/100, 2);
    }

    
    /**
     * This method is used to calculate expected consumption during some time
     * period, given by $start and $end. It uses the average usage the last 14
     * days and the average price for the given time period to estimate the
     * probable expenditures.
     *
     * @return double sum
     */
    public static function getPredictedExpenditureForTimePeriod($custId, $start, $end)
    {
        $periodStart = date("Y-m-d H:i:s", strtotime('-14 days'));
        $now = date('Y-m-d H:i:s');
        $consumption = MeterDataTable::getConsumptionForTimePeriod($custId, $periodStart, $now);
        $averageUsage = $consumption / MeterDataTable::findTimeDifference($periodStart, $now);
        $averagePrice = PriceTable::getAveragePriceForTimePeriod($start, $end);
        $rest = $averageUsage * MeterDataTable::findTimeDifference($start, $end) * $averagePrice;
        return round($rest/100, 2);
    }

    /**
     * This method is used to find the time difference between two datetimes
     * $t1 and $t2 and returns the result in seconds.
     *
     * @return int timeDiff
     */
    public static function findTimeDifference($t1, $t2)
    {
        return abs(strtotime($t2) - strtotime($t1));
    }

    /**
     * This method returns the difference between two expenses given as 
     * the absolute value of the percentual rate
     * 
     * @return double difference
     */
    public static function findDifferenceInExpenditure($e1, $e2)
    {
        if($e2 == 0)
            return "veldig mye";
        return (round((($e1/$e2) - 1) * 100));
    }
}