<?php

/**
 * PriceTable
 * 
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class PriceTable extends Doctrine_Table
{
    /**
     * Returns an instance of this class.
     *
     * @return object PriceTable
     */
    public static function getInstance()
    {
        return Doctrine_Core::getTable('Price');
    }
    
    /**
     * Returns the present price, that is, the price nearest now, in the past.
     *
     * @return double price
     */
    public static function getCurrentPrice(){
        $now = date('Y-m-d H:i:s');
        $currentPrice = Doctrine_Core::getTable('Price')
                ->createQuery('p')
                ->select('p.price')
                ->where('p.starttime <= ?', $now)
                ->orderBy('p.starttime DESC')
                ->limit(1)
                ->fetchOne();
        return $currentPrice->price;
    }
    
    /**
     * This method is used to fetch price data for a given period of time.
     * It returns an array of all the prices and time they start at for the given
     * number of hours.
     *
     * @return array(array(unix_milli, price))
     */
    public static function getPriceForLastHours($hours = 0, $back = false)
    {
        $startTime = new DateTime();
        $startTime = $startTime->format('Y-m-d H:i:s');
        $time = new DateTime();
        $result = array();
        $currentPrice = PriceTable::getCurrentPrice();
        if($back)
        {
            date_sub($time, date_interval_create_from_date_string(floor($hours) ." hours"));
            $endTime = $startTime;
            $startTime = $time->format('Y-m-d H:i:s');
            $firstPriceTime = (strtotime($startTime) + 3600) * 1000;
            $result[] = array($firstPriceTime, PriceTable::getPreviousPriceFromTime($startTime)->price);
        }
        else
        {
            date_add($time, date_interval_create_from_date_string(floor($hours) ." hours"));
            $endTime = $time->format('Y-m-d H:i:s');
            $firstPriceTime = (strtotime($startTime) + 3600) * 1000;
            $result[] = array($firstPriceTime, $currentPrice);
        }
        $prices = Doctrine_Core::getTable('Price')
            ->createQuery('p')
            ->select('UNIX_TIMESTAMP(p.starttime) AS timestamp, p.price')
            ->where('p.starttime < ?', $endTime)
            ->andWhere('p.starttime > ?', $startTime)
            ->orderBy('p.starttime')
            ->execute();
        $price = null;
        foreach ($prices as $price)
        {
            $time = ($price->timestamp + 3600) * 1000;
            $lastPrices = array($time, $price->price);
            $result[] = $lastPrices;
        }
        $time = (strtotime($endTime) + 3600) * 1000;
        if($back)
        {
            $result[] = array($time, $currentPrice);
        }
        else
            $result[] = array($time, $price == null ? $currentPrice : $price->price);
        return $result;
    }

    /**
     * This method is used fetch the price before a given $time. The first price
     * with timestamp less than $time is returned.
     *
     * @return object price
     */
    public static function getPreviousPriceFromTime($time)
    {
        $query = "SELECT * FROM `price` WHERE starttime < '".$time . "' ORDER BY starttime DESC LIMIT 1";
        $connection = Doctrine_Manager::connection();    
        $statement = $connection->execute($query);
        $statement->execute();
        return $statement->fetch(PDO::FETCH_OBJ);
    }

    /**
     * Gets the average price from a period of time.
     *
     * @return double average
     */
    public static function getAveragePriceForTimePeriod($start, $end)
    {
        $avg = Doctrine_Core::getTable('Price')
            ->createQuery('p')
            ->select('AVG(p.price) as average')
            ->where('p.starttime < ?', $end)
            ->andWhere('p.starttime > ?', $start)
            ->fetchOne();
        if(!$avg->average)
        {
            $avg = Doctrine_Core::getTable('Price')
            ->createQuery('p')
            ->select('p.price as average')
            ->orderBy('p.starttime DESC')
            ->fetchOne();
        }
        return $avg->average;
    }

    /**
     * Gets all prices in a time range and returns them as an array of
     * arrays with starttime and price.
     *
     * @return array(array) starttime, price
     */
    public static function getPriceDataForTimePeriod($start, $end)
    {
        $prices = Doctrine_Core::getTable('Price')
            ->createQuery('p')
            ->select('p.starttime, p.price')
            ->where('p.starttime >= ?', $start)
            ->andWhere('p.starttime < ?', $end)
            ->orderBy('p.starttime')
            ->execute();
        $firstPrice = PriceTable::getPreviousPriceFromTime($start);
        $result = array();
        $result[] = array("starttime" => $firstPrice->starttime, "price" => $firstPrice->price);
        foreach ($prices as $price)
        {
            $result[] = array("starttime" => $price->starttime, "price" => $price->price);
        }
        return $result;
    }
}