/*
 * The JavaScript for dealing with the graphing in the "Mitt forbruk"-section.
 * There is a data from and a date to field and two "graphing areas" - one for overview,
 * and one for the main graph. The data is fetched with AJAX-calls and JSON-data is
 * returned. Plotted by the flot library.
 */

var interval = 15;
    var data = [];
    var date = new Date();
    var aMonthAgo = date.getFullYear()+"-"+(zeroPad(date.getMonth()))+"-"+zeroPad(date.getDate())+"T"+zeroPad(date.getHours())+":"+zeroPad(date.getMinutes());
    var now = date.getFullYear()+"-"+(zeroPad(date.getMonth()+1))+"-"+zeroPad(date.getDate())+"T"+zeroPad(date.getHours())+":"+zeroPad(date.getMinutes());
    getInitialData(aMonthAgo, now);
    startGraphing();
    $(function() {
        $("#fromTime").dtpicker();
        $("#toTime").dtpicker();
        $("#fromTime").val(aMonthAgo);
        $("#toTime").val(now);}
    );
function zeroPad(num)
{
    if(num.toString().length == 1) return "0"+num;
    else return num;
}
function startGraphing(){

$(function () {
    
    // setup plot
    function getData(x1, x2) {
        var d = [];
        var xEn = Math.min(x1, x2);
        var xTo = Math.max(x1, x2);
        for (var i = 0; i < data.length; ++i) {
            if(data[i][0] > xEn && data[i][0] <= xTo)
            {
                d.push([data[i][0], data[i][1]]);
            }
        }
        return [
            {label: "kWh", data: d, color: "#166834"}
        ];
    }

  
    var stack = 0, bars = true, lines = false, steps = false;
    var options = {series: {
        stack: stack,
        lines: {show: lines, steps: steps},
        bars: {show: bars, barWidth : interval*60*1000}
        },
        xaxis: {mode: "time", timeformat: "%d.%b %H:%M"},
        yaxis: {tickFormatter: function (v, axis) {return '<span style="color:'+'"#166834"'+ ';">' + v.toFixed(axis.tickDecimals) +" kWh</span>"}},
        selection: {mode: "x"}
    };

    var content = {label: "kWh", color: darkGreen, data: data};
    var plot = $.plot($("#placeholder"), [content], options);
      $.plot($("#placeholder"), [ content ], options);


    // setup overview
    var overview = $.plot($("#overview"), [content], {
        legend: {show: true, container: $("#overviewLegend")},
        stack: stack,
        lines: {show: lines, steps: steps},
        bars: {show: bars, barWidth : interval*60*1000},
        xaxis: {mode: "time", timeformat: "%d.%m"},
        yaxis: {tickFormatter: function (v, axis) {return '<span style="font-size: smaller; color:'+'"#166834"'+ ';">' + v.toFixed(axis.tickDecimals) +"</span>"}},
        selection: {mode: "x"}
    });

    // now connect the two

    $("#placeholder").bind("plotselected", function (event, ranges) {
        // clamp the zooming to prevent eternal zoom
        if (ranges.xaxis.to - ranges.xaxis.from < 0.00001)
            ranges.xaxis.to = ranges.xaxis.from + 0.00001;

        // do the zooming
        plot = $.plot($("#placeholder"), getData(ranges.xaxis.from, ranges.xaxis.to),
                      $.extend(true, {}, options, {
                          xaxis: {min: ranges.xaxis.from, max: ranges.xaxis.to}
                      }));

        // don't fire event on the overview to prevent eternal loop
        overview.setSelection(ranges, true);
    });
    $("#overview").bind("plotselected", function (event, ranges) {
        plot.setSelection(ranges);
    });
});

}

  function getInitialData(start, end)
    {
        data = [];
        $.ajax({
     type: "POST",
     async: false,
     url: "consumptionIntervalSpecified",
     dataType: "json",
     data: "start="+start+"&end=" + end +"&interval="+ interval,
     success: function(results){
      if(results.length > 0)
      {
        var previous = results[0][1];
        var current = results[0][1];
        var stack = 0, bars = true, lines = false, steps = false;
        for (var i = 1; i < results.length; i ++){
          current = results[i][1] - previous;
          previous = results[i][1];
          data.push([results[i][0], current]);
        }
      }
     },
     error: function(a){
         console.log(a);
     }
});
    }
    function reloadWithNewDates(start, end)
    {
        getInitialData(start, end);
        startGraphing();



    var stack = 0, bars = true, lines = false, steps = false;
    var options = {series: {
        stack: stack,
        lines: {show: lines, steps: steps},
        bars: {show: bars, barWidth : interval*60*1000}
        },
        xaxis: {mode: "time", timeformat: "%d.%b %H:%M"},
        yaxis: {tickFormatter: function (v, axis) {return '<span style="color:'+'"#166834"'+ ';">' + v.toFixed(axis.tickDecimals) +" kWh</span>"}},
        selection: {mode: "x"}
    };

    //var dataSelection = getInitialData(168);

    var content = {label: "kWh", color: darkGreen, data: data};
    $.plot($("#placeholder"), [content], options);
      $.plot($("#placeholder"), [ content ], options);


    // setup overview
    $.plot($("#overview"), [content], {
        legend: {show: true, container: $("#overviewLegend")},
        stack: stack,
        lines: {show: lines, steps: steps},
        bars: {show: bars, barWidth : interval*60*1000},
        xaxis: {mode: "time", timeformat: "%d.%b"},
        yaxis: {tickFormatter: function (v, axis) {return '<span style="font-size: smaller; color:'+'"#166834"'+ ';">' + v.toFixed(axis.tickDecimals) +"</span>"}},
        selection: {mode: "x"}
    });
    }