/*
 * Copyright (c) 1997-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package jce.common;

import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import util.Print;
import util.ResourceReader;

/*
 * Utility class for retrieving Elliptic Curve key pair.
 */
public class ECKeyUtil {

    private static final byte[] EC_PUB_KEY_BER_P256 =
            Print.hexStringToByteArray("3059301306072a8648ce3d020106082a8648" +
            "ce3d03010703420004942356a37f3c795286205b0a30232d2d6e5d802c8ef94" +
            "cd8c45f9b0da7af8e54bc4e8677703694e315f67f64ba076f5a86f68b3f2129" +
            "d006c1e242dae41329b7");

    private static final byte[] EC_PRIV_KEY_BER_P256 =
            Print.hexStringToByteArray("3041020100301306072a8648ce3d02010608" +
            "2a8648ce3d03010704273025020101042027ccc92b59bf0c3de407f5244ce9a" +
            "0deec408200b9bd3db11399297246549766");
    /*
     * Get the public key of the pre-generated EC key pair.
     *
     * @return The pre-generated public key.
     * @throws Exception Could not get the public key.
     */
    public static PublicKey getPublic() throws Exception {

        X509EncodedKeySpec pubKeySpec =
                new X509EncodedKeySpec(EC_PUB_KEY_BER_P256);
        KeyFactory factory = KeyFactory.getInstance("EC", "JsafeJCE");
        return factory.generatePublic(pubKeySpec);
    }

    /*
     * Get the private key of the pre-generated EC key pair.
     *
     * @return The pre-generated private key.
     * @throws Exception Could not get the private key.
     */
    public static PrivateKey getPrivate() throws Exception {
        PKCS8EncodedKeySpec privKeySpec =
                new PKCS8EncodedKeySpec(EC_PRIV_KEY_BER_P256);
        KeyFactory factory = KeyFactory.getInstance("EC", "JsafeJCE");
        return factory.generatePrivate(privKeySpec);
    }

    /*
     * Load a key from a file.
     */
    public static PrivateKey loadRawPrivate(String file) throws Exception {
        PKCS8EncodedKeySpec privKeySpec =
                new PKCS8EncodedKeySpec(ResourceReader.readResource(file));
        KeyFactory factory = KeyFactory.getInstance("EC", "JsafeJCE");
        return factory.generatePrivate(privKeySpec);
    }

}
