package crypto;

import javax.crypto.Cipher;
import data.Helper;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;

import jce.common.AESKeyUtil;
import jce.common.JCESample;

import com.rsa.jsafe.provider.SensitiveData;

public class GCM extends JCESample implements MessageProtecter {
	
	private SecretKey secretKey;
	private int frameCounter = 1234567;
	public GCM()
	{
		if(secretKey == null)
		{
			try {
				addJsafeJCE();
				secretKey = AESKeyUtil.getSecretKey();
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}
	}
	// Get correct IV for the decryption part
	public byte[] getPreviousInitializationVector()
	{
		byte[] iv = INITIALIZATION_VECTOR;
		String fcString = "" + (getFrameCounter() - 1);
		// Left fill with zeroes
		for (int i = 0; i < 8-fcString.length(); i++) {
			fcString = "0" + fcString;
		}
		byte[] frame = Helper.hexStringToByteArray(fcString);
		for (int i = 17; i < iv.length; i++) {
			iv[i] = (byte) frame[i-17];
		}
		util.Print.ciphertext(iv);
		return iv;
	}
	
	public byte[] getInitializationVector()
	{
		byte[] iv = INITIALIZATION_VECTOR;
		String fcString = "" + getFrameCounter();
		// Increment for next frame
		frameCounter++;
		// Left fill with zeroes
		for (int i = 0; i < 8-fcString.length(); i++) {
			fcString = "0" + fcString;
		}
		System.out.println(fcString);
		byte[] frame = Helper.hexStringToByteArray(fcString);
		for (int i = 17; i < iv.length; i++) {
			iv[i] = (byte) frame[i-17];
		}
		System.out.println("New IV issued: ");
		util.Print.ciphertext(iv);
		return iv;
	}
	
	public int getFrameCounter() {
		return frameCounter;
	}

	public void setFrameCounter(int frameCounter) {
		if(getFrameCounter() <= frameCounter)
			System.err.println("Frame counter decremented." +
					" Sure this is right. Value before: " 
					+ getFrameCounter() + 
					". Value now: " + frameCounter);
		this.frameCounter = frameCounter;
	}

	private static final byte[] INITIALIZATION_VECTOR = {

        // This is the authentication tag length (12).
        (byte) 0x0c,
        // This is the length of the authenticated data (17).
        (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
        (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x11,

        // This is the GCM IV data used for encryption and decryption.
        // These 12-bytes MUST be unique for a key choice
        // for security reasons.
        (byte) 0x4D, (byte) 0x4D, (byte) 0x4D, (byte) 0x00,
        (byte) 0x00, (byte) 0xBC, (byte) 0x61, (byte) 0x4E,
        (byte) 0x01, (byte) 0x23, (byte) 0x45, (byte) 0x67,
	};

public static void main(String[] args) throws Exception {
    GCMMode aesEncrypter = new GCMMode();
    aesEncrypter.runSample();
}

@Override
public byte[] encryptData(byte[] plainT){
    long a = System.currentTimeMillis();
	// The encryption object.
    Cipher aesEncrypter = null;
    
    // The counter mode parameters.
    IvParameterSpec ivParamSpec;

    // A byte array to hold the encrypted data.
    byte[] ciphertext;

    // This is the plaintext data to encrypt.
    byte[] plaintext = plainT;
    
    // Set the AES IV.
    ivParamSpec = new IvParameterSpec(getInitializationVector());

    try {
        // Get a cipher object for AES encryption with Galois Counter Mode
        // and no padding.  Note that GCM mode is like a stream cipher, so
        // no padding is needed, even if the plaintext length is not a
        // multiple of the block size.
        aesEncrypter = Cipher.getInstance("AES/GCM/NoPadding", "JsafeJCE");

        // Initialize the encryption object.  This call will
        // set up aesEncrypter to perform encryption using AES.
        aesEncrypter.init(Cipher.ENCRYPT_MODE, this.secretKey, ivParamSpec);
        
        // Create some space to hold the results of
        // encrypting the plaintext.  A call to getOutputSize() will
        // return the number of bytes necessary to hold the encrypted
        // data.  This number may be larger than the number that is
        // actually needed.
        ciphertext = new byte[aesEncrypter.getOutputSize(plaintext.length)];

        // First, input the additional authenticated data. The method
        // update() first expects authDataLength bytes of input data.
        // This is processed as the additional authenticated data. The
        // number of bytes expected should have been specified in the
        // ivParamSpec input to init(). There will be no output bytes
        // for input of authenticated data.

        // This is the optional additional data that is authenticated
        // but not encrypted.
        byte[] authenticatedData = getAuthenticatedData();
        int outputLenUpdate = aesEncrypter.update(authenticatedData,
                0, authenticatedData.length, ciphertext, 0);

        // Now, encrypt the data.  The call to update() will
        // encrypt as much data as is possible. Since GCM mode
        // is like a stream cipher, even if the length of
        // plaintext does not contain enough bytes to fill
        // a whole block, the extra bytes will still be encrypted.
        outputLenUpdate += aesEncrypter.update(plaintext, 0,
                plaintext.length, ciphertext, outputLenUpdate);

        // Call doFinal() after passing in all of the data to encrypt.
        // In GCM, doFinal() will output an authentication tag based
        // on the all the input data, apply any necessary padding to the
        // input data, and finish the encryption.
        aesEncrypter.doFinal(ciphertext, outputLenUpdate);
        return (ciphertext);
    }catch (Exception e) {
    	e.printStackTrace();
    	return new byte[2];
	}
    finally {
        // Cryptographic objects should be cleared once they are no longer
        // needed.
        SensitiveData.clear(aesEncrypter);
//        SensitiveData.clear(secretKey);
    }
}

public byte[] decryptData(byte[] cipherT)
{
    // The decryption object.
    Cipher aesDecrypter = null;
    
    // The counter mode parameters.
    IvParameterSpec ivParamSpec;

    // A byte array to hold the encrypted data.
    byte[] ciphertext = cipherT;

    // A byte array to hold the decrypted data.
    byte[] recoveredPlaintext;

    // Set the AES IV.
    ivParamSpec = new IvParameterSpec(getPreviousInitializationVector());
	try{
		// This is security context (SC) padded with authentication key according
		// to standard ==> A = SC || AK
		// Decrypt data.

        // Get a cipher object for AES decryption with Galois/Counter Mode
        // and no padding.
        aesDecrypter = Cipher.getInstance("AES/GCM/NoPadding", "JsafeJCE");

        // Initialize the decryption object.  This call will
        // set up aesDecrypter to perform decryption using AES.
        // Note: the ivParamSpec still contains the same parameters
        //    as used in the encrypt phase (this allows decrypt to match).
        aesDecrypter.init(Cipher.DECRYPT_MODE, this.secretKey, ivParamSpec);
        
        // It's necessary to create some space to hold the results
        // of decrypting the ciphertext.  A call to getOutputSize()
        // will return the number of bytes necessary to hold the
        // decrypted data.  This number may be larger than the number
        // that is actually needed.
        recoveredPlaintext = new byte[aesDecrypter.getOutputSize(ciphertext.length)];

        // First, input the additional authenticated data. The method
        // update() first expects authDataLength bytes of input data.
        // This is processed as the additional authenticated data.
        // The number of bytes expected should have been specified in the
        // ivParamSpec input to init().
        byte[] authenticatedData = getAuthenticatedData();
        int outputLenUpdate = aesDecrypter.update(authenticatedData,
                0, authenticatedData.length, recoveredPlaintext, 0);
        
        // Decrypt the data.  The call to update() will decrypt as
        // much data as is possible.  If not enough bytes are passed in
        // to decrypt a whole block, the extra bytes will be kept for
        // the next call to update(), or a call to doFinal().
        
        outputLenUpdate += aesDecrypter.update(ciphertext, 0,
                ciphertext.length, recoveredPlaintext, outputLenUpdate);
        // After passing in all of the data to decrypt,
        // call doFinal().  In GCM, doFinal() will verify the
        // authenticity of the input data, and will throw an
        // exception if the verification failed.
        // Otherwise, it will finish the decryption.
        aesDecrypter.doFinal(recoveredPlaintext, outputLenUpdate);
        return (recoveredPlaintext);

    }
	catch (Exception e) {
		e.printStackTrace();
		return new byte[2];
	}
	finally {
        // Cryptographic objects should be cleared once they are no longer
        // needed.
        SensitiveData.clear(aesDecrypter);
//        SensitiveData.clear(this.secretKey);
    }
	}

private byte[] getAuthenticatedData() {
	// This is SC (Security control byte. Authenticated encryption = 30 (hex))
	// appended with AK (Authentication key). Section 12.2 Green book.
	return new byte[]{
    		(byte) 0x30, (byte) 0xD0, (byte) 0xD1, (byte) 0xD2,
    		(byte) 0xD3, (byte) 0xD4, (byte) 0xD5, (byte) 0xD6,
    		(byte) 0xD7, (byte) 0xD8, (byte) 0xD9, (byte) 0xDA,
    		(byte) 0xDB, (byte) 0xDC, (byte) 0xDD, (byte) 0xDE,
    		(byte) 0xDF,
    };
}
@Override
public void runSample() throws Exception {
	// TODO Auto-generated method stub
	
}


}
