package dcs;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import data.DB;

import net.Client;
import net.PollServer;

public class DataCollector {
	private DB db;
	private HashMap<String, Meter> meters;
	private ArrayList<Client> clients = new ArrayList<Client>();
	private int numberOfMeters = 0;
	private static final int TIME_BETWEEN_POLLS = 15 *60000; // 15 min
	private int meterSpreadTime = 0;
	private void start()
	{
		db = DB.getDBObject();
		meters = db.getListOfMetersToPoll(true);
		ArrayBlockingQueue<Runnable> taskQueue = new ArrayBlockingQueue<Runnable>(5000);
		ExecutorService threadExecutor = Executors.newFixedThreadPool(meters.size() + 1);
		threadExecutor.execute(new PollServer());
		for (Meter meter : meters.values()) {
			meter.setBeingPolled(true);
			clients.add(new Client(meter));
		}
		long timeA;
		while(true)
		{
			timeA = System.currentTimeMillis();
			numberOfMeters = meters.size();
			if(TIME_BETWEEN_POLLS / numberOfMeters < 3000)
				meterSpreadTime = TIME_BETWEEN_POLLS / numberOfMeters;
			else meterSpreadTime = 3000;
			checkForMeterChanges();
			// Hvis neon er borte, lag hele lista på nytt.
			if(numberOfMeters != numberOfMetersInDB())
			{
				System.out.println("Removing meters from polling list.");
				meters = db.getListOfMetersToPoll(true);
				clients.clear();
				for (Meter meter : meters.values()) {
					meter.setBeingPolled(true);
					clients.add(new Client(meter));
				}
			}
			for (int j = 0; j < clients.size(); j++) 
			{
				if(clients.get(j).getMeter().isBeingPolled())
				{
					threadExecutor.execute(clients.get(j));
						try {
							Thread.sleep(meterSpreadTime);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
				}
			}
			try{
				Thread.sleep(Math.max(TIME_BETWEEN_POLLS + timeA - System.currentTimeMillis(), 0));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	private int numberOfMetersInDB()
	{
		try{
			return db.getNumberOfMeters();
		} catch (SQLException e) {
			System.err.println("Couldn't get number of meters in database.");
			e.printStackTrace();
		}
		return -1;
	}
	
	private void checkForMeterChanges() {
		HashMap<String, Meter> map = db.getListOfMetersToPoll(false);
		Meter ex;
		if(map.size()>0) 
			System.out.println("Assigning new attributes to " + map.size() + " meters.");
		for (Meter meter : map.values())
		{
			meter.setBeingPolled(true);
			ex = meters.put(meter.getMacAddress(), meter);
			if(ex == null)
			{
				clients.add(new Client(meter));
			}
			else
			{
				for (Client cl : clients)
				{
					if(cl.getMeter().getMacAddress().equals(meter.getMacAddress()))
					{
						cl.setMeter(meter);
						break;
					}
				}
			}
		}
	}

	public void clientDied(String hostname)
	{
		meters.get(hostname).setBeingPolled(false);
	}
	
	public static void main(String[] args) {
		DataCollector dc = new DataCollector();
		dc.start();
	}
}
