/*
 * Copyright (c) 1997-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package jce.common;

import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.SecretKeySpec;

/*
 * Utility class for retrieving pre-generated AES secret key.
 */
public class AESKeyUtil {

    // Sample AES 128 bit key.
    private static final byte[] AES_KEY_DATA = {
            (byte) 0x00, (byte) 0x01, (byte) 0x02, (byte) 0x03,
            (byte) 0x04, (byte) 0x05, (byte) 0x06, (byte) 0x07,
            (byte) 0x08, (byte) 0x09, (byte) 0x0A, (byte) 0x0B,
            (byte) 0x0C, (byte) 0x0D, (byte) 0x0E, (byte) 0x0F
    };

    // Sample AES 256 bit key.
    private static final byte[] AES_KEY_DATA_256 = {
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00,
            (byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x00
    };

    /*
     * Get a pre-generated AES secret key.
     *
     * @return The pre-generated secret key.
     * @throws Exception Could not get the secret key.
     */
    public static SecretKey getSecretKey() throws Exception {

        SecretKeyFactory skf;
        skf = SecretKeyFactory.getInstance("AES", "JsafeJCE");
        return skf.generateSecret(new SecretKeySpec(AES_KEY_DATA, "AES"));
    }

    /*
     * Get a pre-generated AES 256-bit secret key.
     *
     * @return The pre-generated secret key.
     * @throws Exception Could not get the secret key.
     */
    public static SecretKey getSecretKey256() throws Exception {

        SecretKeyFactory skf;
        skf = SecretKeyFactory.getInstance("AES", "JsafeJCE");
        return skf.generateSecret(new SecretKeySpec(AES_KEY_DATA_256, "AES"));
    }
}
