/*
 * Copyright (c) 1997-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package jce.common;

import java.security.Provider;
import java.security.SecureRandom;
import java.security.Security;

import util.Print;

/*
 * This class provides the implementation of some helper methods that are
 * useful for sample code.
 * Any class extending <code>JCESample</code> needs to implement the abstract
 * method <code>runSample()</code>. This method should do the work of the class.
 */
public abstract class JCESample {

    private static float fJreVersion;

    protected static final String UNLIMITED_POLICY_FILES_REQUIRED =
            "This sample requires the unlimited strength " +
            "jurisdiction policy files. Please download from " +
            "the following URL: \n" +
            "JDK 5.0   - " +
            "http://java.sun.com/javase/downloads/index_jdk5.jsp\n" +
            "JDK 6.0   - " +
            "http://java.sun.com/javase/downloads/index.jsp\n\n" +
            "To install the Unlimited Jurisdiction Policy Files:\n" +
            "  - Extract the local_policy.jar and US_export_policy.jar files" +
            " from the downloaded zip file.\n" +
            "  - Copy local_policy.jar and US_export_policy.jar to the " +
            "<jdk install directory>/jre/lib/security directory, " +
            "over-writing the files of the same name.";

    /*
     * Abstract method that provides the functionality of the application.
     * A sample program needs to implement the <code>runSample()</code> method
     * in order to properly extend this class.
     *
     * @throws Exception On failure.
     */
    public abstract void runSample() throws Exception;

    /*
     * Prints out the list of the currently registered providers.
     */
    protected void printRegisteredProviders() {

        // Print out a list of the current providers.
        Provider[] currentProviders = Security.getProviders();
        Print.println("Current providers:  ");
        for (int index = 0; index < currentProviders.length; index++) {
            Print.println(currentProviders[index].getName() + " " +
                    currentProviders[index].getVersion());
            Print.println("  " + currentProviders[index].getInfo());
        }
        Print.println();
    }

    /**
     * Adds the JsafeJCE provider if it has not been added already.
     *
     * @throws Exception On failure.
     */
    protected void addJsafeJCE() throws Exception {

        // Remove provider if it's already registered, or insert will fail.
        Security.removeProvider("JsafeJCE");

        // Create a new provider object for the JsafeJCE provider.
        Provider jsafeProvider = new com.rsa.jsafe.provider.JsafeJCE();

        // Register provider in 1st position.
        int position = Security.insertProviderAt(jsafeProvider, 1);
        if (position != 1) {
            throw new RuntimeException(
                    "Failed to insert provider at first position");
        }
    }

    protected byte[] generateSeed() {
        SecureRandom seeder = new SecureRandom();
        return seeder.generateSeed(20);
    }

}
