package net;

import java.io.*;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.UnknownHostException;
import java.sql.SQLException;

import data.DB;
import data.HDLCData;
import data.XMLParser;
import data.Helper;
import dcs.DataCollector;
import dcs.Meter;

public class Client implements Runnable {
	private static final int CONNECTION_TIMEOUT = 2500;
	protected Meter meter;
	private InetAddress addr; 
	private OutputStream out;
	protected DataOutputStream dos;
	private InputStream in;
	protected DataInputStream dis;
	private Socket myClient;
	private DB db;
	protected byte[] rawData = new byte[200];
	protected byte[] apdu;
	protected String activePowerValue = "";
	protected int receivedDataLength = 0;
	protected static int responseHeaderLength = 11;
	protected static int responseTrailingBytesLength = 3;
	private int connectionAttempts = 0;
	public Meter getMeter() {
		return meter;
	}
	public void setMeter(Meter meter) {
		this.meter = meter;
	}
	public Client(Meter m)
	{
		meter = m;
		db = DB.getDBObject();
	}
	
	private void setUpTCPConnection() throws IOException, UnknownHostException
	{
		if(myClient == null || !myClient.isConnected() || myClient.isClosed())
		{
			addr = InetAddress.getByName(meter.getIpAddress());
			myClient = new Socket();
			myClient.connect(new InetSocketAddress(addr, meter.getPort()), CONNECTION_TIMEOUT);
			out = myClient.getOutputStream();
			in = myClient.getInputStream();
			dos = new DataOutputStream(out);
			dis = new DataInputStream(in);
		}
		
	}
	
	private void resetConnection()
	{
		try{
			if(!myClient.isClosed())
			{
				myClient.close();
				dos.close();
				dis.close();
				System.out.println("Closed down connection");
			}
		} catch (IOException e) {
			System.err.println("Did not manage to reset connection");
		}
	}
	
	public void collectData()
	{
		double activePower = -1.0;
		try{
			setUpTCPConnection();
		    setUpHDLCConnection();
			sendAssociationRequest();
			activePower = getCurrentPower();
			saveDataToDB(activePower);
			resetConnection();
		} catch (UnknownHostException e) {
			System.err.println("Failed to locate " + meter.getIpAddress());
			e.printStackTrace();
		} catch (IOException e) {
			System.err.println("Connection fail. Couldn't connect to " + meter.getIpAddress() + " at port " + meter.getPort());
			connectionAttempts++;
			e.printStackTrace();
		} catch (SQLException e) {
			System.err.println("Failed to connect to database");
			e.printStackTrace();
		}
	}
	
	protected void saveDataToDB(double activePower) throws SQLException {
		if(activePower != -1.0)
		{
			synchronized (db) {
				db.insertActivePower(activePower);
			}
		}
		
	}
	private void setUpHDLCConnection()
	{
		try {
			dos.write(HDLCData.openHdlcConnection, 0, HDLCData.openHdlcConnection.length);
			receivedDataLength = dis.read(rawData);
			apdu = new byte[receivedDataLength];
			apdu = Helper.get(rawData, responseHeaderLength, receivedDataLength-responseHeaderLength - responseTrailingBytesLength);
		} catch (IOException e) {
			System.err.println("Failed to open HDLC connection");
			e.printStackTrace();
		}
	}
	
	private void sendAssociationRequest() throws IOException
	{
		dos.write(HDLCData.associationRequestNoSecurity, 0, HDLCData.associationRequestNoSecurity.length);
		receivedDataLength = dis.read(rawData);
		apdu = new byte[receivedDataLength];
		apdu = Helper.get(rawData, responseHeaderLength, receivedDataLength-responseHeaderLength-responseTrailingBytesLength);
	}
	
	private double getCurrentPower() throws IOException
	{
		dos.write(HDLCData.getActivePower, 0, 
				HDLCData.getActivePower.length);
		System.out.println("Polling meter on " + meter.getIpAddress() + " for active power.");
		receivedDataLength = dis.read(rawData);
		apdu = new byte[receivedDataLength];
		apdu = Helper.get(rawData, responseHeaderLength, receivedDataLength-responseHeaderLength-responseTrailingBytesLength);
		activePowerValue = XMLParser.getSingleValueFromElementName(XMLParser.pduToXml(apdu), "DoubleLongUnsigned", "Value");
		double activePower = Helper.stringOfBytesToInt(activePowerValue, 0);
		return activePower;
	}

	public void run()
	{
		System.out.println("new client started for " + meter.getIpAddress());
		collectData();
	}
}
