/*
 * Copyright (c) 1999-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package util;

/**
 * Utility class to convert byte array to hexadecimal string and vice versa.
 */
public final class ConvertUtil {

    private ConvertUtil() {
        // Avoid instantiation
    }
    /**
     * Converts a hexadecimal string to a byte array.
     *
     * @param hexString A hexadecimal string.
     * @return A byte array representation of the hex string.
     */
    public static byte[] hexStringToByteArray(String hexString) {
        byte[] bytes = new byte[hexString.length() / 2];
        for (int i = 0; i < bytes.length; ++i) {
            bytes[i] = (byte)
                    (Character.digit(hexString.charAt(i * 2), 16) << 4 |
                    Character.digit(hexString.charAt(i * 2 + 1), 16));
        }
        return bytes;
    }


    /**
     * Converts a byte array to a hexadecimal string.
     *
     * @param buffer   With byte array representation of the hex string.
     * @return A hexadecimal hex string.
     */
    public static String byteArrayToHexString(byte[] buffer) {
        char[] chars = new char[buffer.length * 2];
        for (int i = 0; i < buffer.length; ++i) {
            chars[i * 2] = Character.forDigit(buffer[i] >>> 4 & 0xf, 16);
            chars[i * 2 + 1] = Character.forDigit(buffer[i] & 0xf, 16);
        }
        return new String(chars);
    }
}
