/*
 * Copyright (c) 1997-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package util;

/*
 * Utility class for printing data to standard output.
 */
public class Print {

    public static final String SIGNATURE_NOT_VERIFIED =
            "Failure: Signature was not verified.";

    public static final String RECOVERED_PLAINTEXT_DOES_NOT_MATCH_ORIGINAL =
            "Failure: Recovered plaintext does not match original plaintext.";

    public static final String UNSUCCESSFUL_SHARED_SECRETS_DO_NOT_MATCH =
            "Unsuccessful, shared secrets do not match.";

    /*
     * Print the plaintext data to standard output.
     *
     * @param data The plaintext.
     * @param len The length of plaintext.
     */
    public static void plaintext(byte[] data, int len) {
        println("Plaintext: ");
        printBuffer(data, len);
    }

    /*
     * Print the plaintext data to standard output.
     *
     * @param data The plaintext.
     */
    public static void plaintext(byte[] data) {
        plaintext(data, data.length);
    }

    /*
     * Print the ciphertext data to standard output.
     *
     * @param data The ciphertext.
     * @param len The length of ciphertext.
     */
    public static void ciphertext(byte[] data, int len) {
        println("Ciphertext: ");
        printBuffer(data, len);
    }

    /*
     * Print the ciphertext data to standard output.
     *
     * @param data The ciphertext.
     */
    public static void ciphertext(byte[] data) {
        ciphertext(data, data.length);
    }

    /*
     * Print the generated random bytes to standard output.
     *
     * @param data The generated random bytes.
     */
    public static void generatedRandomBytes(byte[] data) {
        println("Generated Random Bytes: ");
        printBuffer(data, data.length);
    }

    /*
     * Print the recovered plaintext data to standard output.
     *
     * @param data The recovered plaintext.
     * @param len The length of the recovered plaintext.
     */
    public static void recoveredPlaintext(byte[] data, int len) {
        println("Recovered Plaintext: ");
        printBuffer(data, len);
    }

    /*
     * Print the recovered plaintext data to standard output.
     *
     * @param data The recovered plaintext.
     */
    public static void recoveredPlaintext(byte[] data) {
        recoveredPlaintext(data, data.length);
    }

    /*
     * Print the message data to standard output.
     *
     * @param data The message.
     * @param len The length of message.
     */
    public static void message(byte[] data, int len) {
        println("Message: ");
        printBuffer(data, len);
    }

    /*
     * Print the message data to standard output.
     *
     * @param data The message.
     */
    public static void message(byte[] data) {
        message(data, data.length);
    }

    /*
     * Print the message data to standard output.
     *
     * @param data The message.
     * @param len The length of message.
     */
    public static void digest(byte[] data, int len) {
        println("Digest: ");
        printBuffer(data, len);
    }

    /*
     * Print the message data to standard output.
     *
     * @param data The message.
     */
    public static void digest(byte[] data) {
        digest(data, data.length);
    }

    /*
     * Print the signature data to standard output.
     *
     * @param data The signature.
     * @param len The length of signature.
     */
    public static void signature(byte[] data, int len) {
        println("Signature: ");
        printBuffer(data, len);
    }

    /*
     * Print the signature data to standard output.
     *
     * @param data The signature.
     */
    public static void signature(byte[] data) {
        signature(data, data.length);
    }

    /*
     * Print the MAC data to standard output.
     *
     * @param data The MAC.
     * @param len The length of MAC.
     */
    public static void mac(byte[] data, int len) {
        println("MAC: ");
        printBuffer(data, len);
    }

    /*
     * Print the MAC data to standard output.
     *
     * @param data The MAC.
     */
    public static void mac(byte[] data) {
        mac(data, data.length);
    }

    /*
     * Print the salt data to standard output.
     *
     * @param data The salt data.
     */
    public static void salt(byte[] data) {
        println("Salt: ");
        printBuffer(data);
    }

    /*
     * Print the IV data to standard output.
     *
     * @param data The IV data.
     */
    public static void iv(byte[] data) {
        println("The " + data.length + " bytes of the IV are:");
        printBuffer(data);
    }

    /*
     * Prints out the key data along with the given
     * description of what part of the key the data is.
     *
     * @param description A String describing the key data.
     * @param data The key data.
     */
    public static void keyData(String description, byte[] data) {
        println(description + ":");
        printBuffer(data);
    }

    /*
     * Prints out to standard out the current JDK version.
     */
    public static void jdkVersion() {
        println("JDK version is " + System.getProperty("java.version"));
    }

    /*
     * Prints out a message to standard out indicating a successful ending.
     */
    public static void successfulEnding() {
        println("Successful Ending.");
    }

    /*
     * Prints out a message to standard out indicating sample has started.
     *
     * @param sampleDescription The description of the sample.
     *
     */
    public static void beginSample(String sampleDescription) {
        println("Beginning " + sampleDescription + " Sample.");
    }

    /*
     * Prints out a message to standard out indicating a PKCS#11 session
     * initialized successfully.
     *
     * @param libName The PKCS#11 library name.
     */
    public static void sessionInitialized(String libName) {
        println("Session successfully initialized on " + libName);
    }

    /*
     * Prints out a message to standard out indicating that the
     * recovered plaintext does not match original plaintext.
     */
    public static void recoveredPlaintextDoesNotMatchOriginal() {
        println(RECOVERED_PLAINTEXT_DOES_NOT_MATCH_ORIGINAL);
    }

    /*
     * Prints out a message to standard out indicating that
     * signature verification failed.
     */
    public static void signatureVerifyFailed() {
        println(SIGNATURE_NOT_VERIFIED);
    }

    /*
     * Print the specified text to standard output.
     *
     * @param textToPrint The text to be printed.
     */
    public static void print(String textToPrint) {
        System.out.print(textToPrint);
    }

    /*
     * Print a blank line to standard output.
     */
    public static void println() {
        System.out.println();
    }

    /*
     * Print the specified text to standard output.
     *
     * @param textToPrint The text to print.
     */
    public static void println(String textToPrint) {
        System.out.println(textToPrint);
    }

    /*
     * Given a <code>byte</code> array, print out the hex-encoding of that data,
     * in a visually pleasing format.
     *
     * @param byteArray The <code>byte</code> array to be printed.
     */
    public static void printBuffer(byte[] byteArray) {
        printBuffer(byteArray, 0, byteArray.length);
    }

    /*
     * Given a <code>byte</code> array, print the first length bytes of
     * that <code>byte</code> array.
     *
     * @param byteArray The <code>byte</code> array containing the data to be printed.
     * @param length    The amount of data to be printed.
     */
    public static void printBuffer(byte[] byteArray, int length) {
        printBuffer(byteArray, 0, length);
    }

    /*
     * Given some part of a <code>byte</code> array, print the
     * hex-encoding of that data in a visually pleasing format.
     *
     * @param byteArray The <code>byte</code> array containing data to print.
     * @param offset    The starting location of the data to print.
     * @param length    The amount of data to print.
     */
    public static void printBuffer(byte[] byteArray, int offset, int length) {
        StringBuffer textLine = new StringBuffer("                ");
        print("  0000: ");
        for (int i = 0; i < length; i++) {
            if (i % 16 == 0 && i != 0) {
                println("[" + textLine + "]");
                print("  " + hexString(i, 4) + ": ");
                for (int j = 0; j < 16; j++) {
                    textLine.setCharAt(j, ' ');
                }
            }
            print(hexString((int) byteArray[i + offset], 2) + " ");
            if (byteArray[i + offset] < 32 || byteArray[i + offset] > 127 ||
                    byteArray[i + offset] == 0x7f) {
                textLine.setCharAt(i % 16, '.');
            } else {
                textLine.setCharAt(i % 16, (char) byteArray[i + offset]);
            }
        }
        if (length % 16 != 0 || length == 0) {
            for (int i = 0; i < 16 - length % 16; i++) {
                print("   ");
            }
        }
        println("[" + textLine + "]");
    }

    private static String hexString(int value, int padding) {
        String hexString = "0123456789ABCDEF";
        StringBuffer tempString = new StringBuffer
                ("                                                                              ".substring(0, padding));
        int offset = padding - 1;

        for (int i = 0; i < padding; i++) {
            tempString.setCharAt(offset - i,
                    hexString.charAt(value >> i * 4 & 0xF));
        }
        return tempString.toString();
    }

    /**
     * Converts a hexadecimal string to a byte array.
     * <p/>
     *
     * @param hexString A hexadecimal string.
     * @return A byte array representation of the hex string.
     */
    public static byte[] hexStringToByteArray(String hexString) {
        byte[] bytes = new byte[hexString.length() / 2];
        for (int i = 0; i < bytes.length; ++i) {
            bytes[i] = (byte) (Character.digit(hexString.charAt(i * 2), 16) << 4 |
                    Character.digit(hexString.charAt(i * 2 + 1), 16));
        }
        return bytes;
    }

    /*
     * Print the data in byteArray straight to the System.out stream.
     *
     * @param byteArray The byte array to print.
     * @param arrayLen The number of bytes to print.
     */
    public static void printBufferAsASCII(byte[] byteArray, int arrayLen) {
        for (int i = 0; i < arrayLen; i++) {
            print(String.valueOf((char) byteArray[i]));
        }
    }

    /*
     * Compare the first length bytes in two buffers, returning false
     * if something doesn't match.
     *
     * @param bufferA The first array to compare.
     * @param bufferB The second array to compare.
     * @param length  The length of the arrays to compare.
     * @return <code>true</code> if the arrays match,
     *         <code>false</code> otherwise.
     */
    public static boolean compareBuffers(byte[] bufferA, byte[] bufferB,
            int length) {
        for (int i = 0; i < length; i++) {
            if (bufferA[i] != bufferB[i]) {
                return false;
            }
        }

        return true;
    }
}
