/*
 * Copyright (c) 1999-2010 EMC Corporation. All rights reserved.
 *
 * This file shall only be used to demonstrate how to interface to an
 * EMC Corporation licensed development product.
 *
 * You have a royalty-free right to use, reproduce and distribute this
 * demonstration file, provided that you agree that EMC Corporation has no
 * warranty, implied or otherwise, or liability for this demonstration file
 * (including any modified version).  This software is provided "as is"
 * without warranties or representations of any kind. EMC Corporation
 * disclaims all conditions and warranties, statutory and otherwise, both
 * express and implied, with respect to the software, its quality and
 * performance, including but not limited to, all implied warranties of
 * merchantability, fitness for a particular purpose, title and
 * noninfringement of third party rights. Without limiting the foregoing,
 * EMC Corporation does not warrant that the software is error-free or that
 * errors in the product will be corrected. You agree that EMC Corporation
 * shall not be liable for any direct, indirect, incidental, special,
 * consequential, punitive or other damages whatsoever resulting from your use
 * of this software or any modified version.
 *
 */
package util;

import java.io.IOException;
import java.io.InputStream;

/**
 * Utility class to read resource from Jar file or from file system.
 */
public final class ResourceReader {

    private static final int MAX_LEN = 1024;

    private ResourceReader() {
        // Prevent instantiation.
    }

    public static byte[] readResource(String resName) throws IOException {
        InputStream fileIn = getResourceAsStream(resName);

        try {
            byte[] buf = new byte[MAX_LEN];
            int pos = 0;
            int n;
            while ((n = fileIn.read(buf, pos, buf.length - pos)) != -1) {

                pos += n;
                if (pos == buf.length) {

                    // There is no more space in the buffer.  Double the size.
                    byte[] oldBuf = buf;
                    buf = new byte[oldBuf.length * 2];
                    System.arraycopy(oldBuf, 0, buf, 0, oldBuf.length);
                }
            }
            byte[] resBytes = new byte[pos];
            System.arraycopy(buf, 0, resBytes, 0, pos);
            return resBytes;
        } finally {
            fileIn.close();
        }
    }

    public static InputStream getResourceAsStream(String s) {
        return ResourceReader.class.getResourceAsStream(s);
    }
}
